from pongpy.models.pos import Pos


class Ball:
    size: int
    pos: Pos
    vx: float
    vy: float

    def __init__(self, size: int, pos: Pos, vx: float, vy: float):
        self.size = size
        self.pos = pos
        self.vx = vx
        self.vy = vy

    def updated(self):
        return Pos(self.pos.x + self.vx, self.pos.y + self.vy)

    def __str__(self):
        return f'px: {self.pos.x} py: {self.pos.y} vx: {self.vx} vy: {self.vy}'



from logging import getLogger

import pyxel

from pongpy.controllers.board import Board
from pongpy.definitions import PADDING, HEIGHT, WIDTH, BOARD_WIDTH, BOARD_HEIGHT, SET_POINT, MATCH_POINT, FRAME_RATE
from pongpy.interfaces.team import Team
from pongpy.models.color import Color

logger = getLogger(__name__)


class PongMatch:
    team1: Team
    team2: Team
    team1_set_count: int
    team2_set_count: int
    turn: bool

    def __init__(self, team1: Team, team2: Team):
        self.team1 = team1
        self.team2 = team2
        self.team1_set_count = 0
        self.team2_set_count = 0
        self.turn = 1

    def start(self):
        while self.team1_set_count < MATCH_POINT and self.team2_set_count < MATCH_POINT:
            result = {}  # 結果受け渡し用参照オブジェクト
            if self.turn > 0:
                Pong(self.team1, self.team2, result=result)
                if result['win_left']:
                    self.team1_set_count += 1
                else:
                    self.team2_set_count += 1
            else:
                Pong(self.team2, self.team1, result=result)
                if result['win_left']:
                    self.team2_set_count += 1
                else:
                    self.team1_set_count += 1
            self.turn *= -1
        win_team = self.team1 if self.team1_set_count > self.team2_set_count else self.team2
        print(f'Winner: {win_team.name}')


class Pong:
    board: Board
    left_team: Team
    right_team: Team
    result: dict
    turn: bool
    games = []

    def __init__(self, left_team: Team, right_team: Team, result: dict):
        pyxel.init(WIDTH, HEIGHT, fps=FRAME_RATE)
        self.left_team = left_team
        self.right_team = right_team
        self.board = Board(BOARD_WIDTH, BOARD_HEIGHT, PADDING, PADDING, left_team=self.left_team, right_team=self.right_team)
        self.result = result
        pyxel.run(self.update, self.draw)

    def update(self):

        self.board.update()

        # 1ゲーム終了条件
        if self.board.p1.score >= SET_POINT or self.board.p2.score >= SET_POINT:
            print(f'{self.board.p1.score_label} {self.board.p2.score_label}')
            self.result.update({
                'win_left': self.board.p1.score > self.board.p2.score,
                'p1': self.board.p1.score,
                'p2': self.board.p2.score
            })
            pyxel.quit()

    def draw(self):
        pyxel.cls(Color.BLACK.value)
        self.board.draw()
        pyxel.text(PADDING * 2, HEIGHT - PADDING * 2, ''.join('o' if x else 'x' for x in self.games), Color.GRAY.value)



import importlib

import click

from pongpy.pong import PongMatch


@click.command()
@click.argument('team1', default='pongpy.teams.random_team:RandomTeam')
@click.argument('team2', default='pongpy.teams.follow_team:FollowTeam')
def cmd(team1, team2):
    t1 = dynamic_import(team1)
    t2 = dynamic_import(team2)
    PongMatch(t1(), t2()).start()


def main():
    cmd()


def dynamic_import(name):
    module_name, class_name = name.split(':')
    module = importlib.import_module(module_name)
    klass = getattr(module, class_name)
    return klass


if __name__ == '__main__':
    main()



from typing import NamedTuple

from pongpy.definitions import BALL_SIZE, ATK_SIZE, ATK_DELTA_LIMIT, DEF_SIZE, DEF_DELTA_LIMIT, BAR_WIDTH


class GameInfo(NamedTuple):
    """
    ゲームの設定値が入っているオブジェクト
    """
    width: int  # Pong フィールドの縦幅
    height: int  # Pong フィールドの横幅
    ball_size = BALL_SIZE  # ボールの大きさ
    atk_size = ATK_SIZE  # atk の縦幅
    atk_return_limit = ATK_DELTA_LIMIT  # 1 フレームでの atk の最大移動距離
    def_size = DEF_SIZE  # def の縦幅
    def_return_limit = DEF_DELTA_LIMIT  # 1 フレームでの def の最大移動距離
    bar_width = BAR_WIDTH  # atk, def の横幅



from os import environ
BOARD_WIDTH = int(environ.get('PONGPY_BOARD_WIDTH', 192))  # PONGの横幅
BOARD_HEIGHT = int(environ.get('PONGPY_BOARD_HEIGHT', 128))  # PONGの縦幅
PADDING = int(environ.get('PONGPY_PADDING', 3))  # 余白サイズ
TEXT_SIZE = int(environ.get('PONGPY_TEXT_SIZE', 16))
WIDTH = BOARD_WIDTH + PADDING * 2  # 画面横幅
HEIGHT = BOARD_HEIGHT + PADDING + TEXT_SIZE  # 画面縦幅

BALL_SIZE = int(environ.get('PONGPY_BALL_SIZE', 4))  # Ball の大きさ
ATK_SIZE = int(environ.get('PONGPY_ATK_SIZE', 8))  # ATK bar の長さ
DEF_SIZE = int(environ.get('PONGPY_DEF_SIZE', 24))  # DEF bar の長さ
MAX_VY = int(environ.get('MAX_VY', 8))  # ボールの縦方向の最大速度
ATK_DELTA_LIMIT = int(environ.get('PONGPY_ATK_DELTA_LIMIT', 2))  # ATK の移動制限
DEF_DELTA_LIMIT = int(environ.get('PONGPY_DEF_DELTA_LIMIT', 1))  # DEF の移動制限
BAR_WIDTH = int(environ.get('PONGPY_BAR_WIDTH', 2))  # bar の横幅
SET_POINT = int(environ.get('PONGPY_SET_POINT', 5))  # 1ゲームの勝利点
MATCH_POINT = int(environ.get('PONGPY_MATCH_POINT', 2))  # 何ゲーム先取で勝利か
FRAME_RATE = int(environ.get('PONGPY_FRAME_RATE', 100))  # FPS



import abc

from pongpy.models.game_info import GameInfo
from pongpy.models.state import State


class Team(metaclass=abc.ABCMeta):
    """
    チームを作るためのインターフェース。
    これを継承して自チームを作ってください。
    """

    @property
    @abc.abstractmethod
    def name(self) -> str:
        """
        Team名を返す関数
        :return name: 半角英数字、16字以内
         """
        return self.__class__.__name__

    @abc.abstractmethod
    def atk_action(self, info: GameInfo, state: State) -> int:
        """
        毎フレームの atk の行動を決定する関数。

        :param info: ゲームに関する設定値が入っているオブジェクト
        :param state: ボールや自機の位置など現在のステータスが入っているオブジェクト
        :return y: 縦方向にどれくらい動くか -atk_return_limit <= y <= atk_return_limit
        """
        return 0

    def def_action(self, info: GameInfo, state: State) -> int:
        """
        毎フレームの def の行動を決定する関数。

        :param info: ゲームに関する設定値が入っているオブジェクト
        :param state: ボールや自機の位置など現在のステータスが入っているオブジェクト
        :return y: 縦方向にどれくらい動くか -def_return_limit <= y <= def_return_limit
        """
        return 0



import pyxel

from pongpy.interfaces.team import Team
from pongpy.models.game_info import GameInfo
from pongpy.models.state import State


class ManualTeam(Team):
    """
    デバッグ用の手動操作チーム。
    Pyxel を直接読んでいるのでデバッグ用と以外では利用しない。
    """
    @property
    def name(self) -> str:
        return 'Manual'

    def atk_action(self, info: GameInfo, state: State) -> int:
        if pyxel.btn(pyxel.KEY_I):
            return -info.atk_return_limit
        if pyxel.btn(pyxel.KEY_K):
            return info.atk_return_limit
        return 0

    def def_action(self, info: GameInfo, state: State) -> int:
        if pyxel.btn(pyxel.KEY_W):
            return -info.def_return_limit
        if pyxel.btn(pyxel.KEY_S):
            return info.def_return_limit
        return 0



import pyxel

from pongpy.interfaces.team import Team
from pongpy.models.game_info import GameInfo
from pongpy.models.state import State


class DqnTeam(Team):
    """ デバッグ用、手動操作チーム """
    @property
    def name(self) -> str:
        return 'dqn'

    def atk_action(self, info: GameInfo, state: State) -> int:
        if pyxel._app._capture_images[0]:
            print(pyxel._app._get_capture_image(0))
            print(state.mine_team.score)
            print(state.enemy_team.score)
        if pyxel.btn(pyxel.KEY_I):
            return -info.atk_return_limit
        if pyxel.btn(pyxel.KEY_K):
            return info.atk_return_limit
        return 0

    def def_action(self, info: GameInfo, state: State) -> int:
        if pyxel.btn(pyxel.KEY_W):
            return -info.def_return_limit
        if pyxel.btn(pyxel.KEY_S):
            return info.def_return_limit
        return 0
